//
//  ContentView.swift
//  ios-demo
//
//  Created by photographb on 2023/7/17.
//

import SwiftUI
import WebKit
import Foundation


class MyViewController: UIViewController, WKUIDelegate, WKScriptMessageHandler {
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        if message.name == "getVerifyResult", let messageBody = message.body as? String {
            if let data = messageBody.data(using: .utf8) {
                do {
                    if let jsonObject = try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any] {
                        // 解析JSON
                        let captchaVerifyParam = jsonObject["data"]as! String;
                        let callbackName = jsonObject["callback"] as! String;
                        // 用captchaVerifyParam请求后端接口拿到请求结果
                        let response: [String: Any] = [
                            "captchaResult": true,
                            "bizResult": true
                        ]; // result可以转换为一个JSON字符串，注意传给H5之后的数据格式转换
                        do {
                            let jsonData = try JSONSerialization.data(withJSONObject: response, options: []);
                            // 将JSON数据转换为字符串
                            if let verifyResult = String(data: jsonData, encoding: .utf8) {
                                // 执行回调，传递结果回H5页面
                                sendJSONToWebView(jsonString: verifyResult, callbackName: callbackName);
                                // 根据result在App侧来执行业务自定义提示逻辑，H5中的onBizResultCallback可以置为空函数
                            }
                        } catch {
                            print("Error serializing JSON: \(error)")
                        }
                    }
                } catch {
                    print("Failed to parse JSON: \(error)")
                }
            }
        }
        // 关闭webview
        if message.name == "closeWebView" {
            webView.removeFromSuperview();
        }
    }
    
    func sendJSONToWebView(jsonString: String, callbackName: String) {            // JavaScript 函数的调用
        let javascriptFunction = "\(callbackName)('\(jsonString)');"
        
        webView.evaluateJavaScript(javascriptFunction) { (result, error) in
            if let error = error {
                print("Error calling JavaScript function: \(error)")
            } else {
                print("Sent JSON to WebView")
            }
        }
    }
    
    
    var webView: WKWebView!
    
    override func loadView() {
        let javascript = "var meta = document.createElement('meta'); meta.setAttribute('name', 'viewport'); meta.setAttribute('content', 'width=device-width'); document.getElementsByTagName('head')[0].appendChild(meta)";
        let configuration = WKWebViewConfiguration();
        let userScript = WKUserScript(source: javascript, injectionTime: .atDocumentEnd, forMainFrameOnly: true);
        let usercontroller = WKUserContentController();
        usercontroller.addUserScript(userScript);
        configuration.userContentController = usercontroller;
        // 添加HTML页面js的调用方法，这里默认添加的方法名称为getVerifyResult，可自行按需更改
        configuration.userContentController.add(self, name: "getVerifyResult");
        configuration.userContentController.add(self, name: "closeWebView");
        webView = WKWebView(frame: .zero, configuration: configuration);
        webView.uiDelegate = self;
        webView.configuration.preferences.javaScriptEnabled = true;
        // 开启webview的devtools
        webView.configuration.preferences.setValue(true, forKey: "developerExtrasEnabled");
        if #available(iOS 16.4, *) { webView.isInspectable = true; }
        view = webView
    }


    override func viewDidLoad() {
        super.viewDidLoad()
        if let htmlURL = Bundle.main.url(forResource: "index", withExtension: "html") {
            webView.loadFileURL(htmlURL, allowingReadAccessTo: htmlURL.deletingLastPathComponent())
        }
    }
    
    
    func webView(_ webView: WKWebView,
                 runJavaScriptAlertPanelWithMessage message: String,
                 initiatedByFrame frame: WKFrameInfo,
                 completionHandler: @escaping () -> Void) {
        let alert = UIAlertController(title: nil, message: message, preferredStyle: .alert)
        let title = NSLocalizedString("OK", comment: "OK Button")
        let ok = UIAlertAction(title: title, style: .default) { (action: UIAlertAction) -> Void in
            alert.dismiss(animated: true, completion: nil)
        }
        alert.addAction(ok)
        present(alert, animated: true)
        completionHandler()
    }
}

struct MyView: UIViewControllerRepresentable {
    typealias UIViewControllerType = MyViewController
    
    func makeUIViewController(context: Context) -> MyViewController {
        let vc = MyViewController()
        // Do some configurations here if needed.
        return vc
    }

    func updateUIViewController(_ uiViewController: MyViewController, context: Context) {
        // Updates the state of the specified view controller with new information from SwiftUI.
    }
}

struct ContentView: View {
    @State var isPresented = false
    
    var body: some View {
        MyView().ignoresSafeArea()
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
